-- prepared by Steve Stedman
-- http://SteveStedman.com
-- twitter:  @SqlEmt
-- MySQL JOIN Types




-- drop and recreate our databse if we need to
DROP DATABASE IF EXISTS QueryTraining;
CREATE DATABASE QueryTraining;
USE QueryTraining;




-- Create some tables to match up with the Venn Diagrams
CREATE TABLE Table1
(
	id   INTEGER NOT NULL AUTO_INCREMENT,
	Name varchar(100) NOT NULL,
    fk   INTEGER NULL,
    fk_table3 INTEGER NULL,
PRIMARY KEY (id)
) ;

CREATE TABLE Table2
(
	id   INTEGER NOT NULL,
	FavoriteColor varchar(100) NOT NULL
) ;
-- notice... Table2 is more real world, kinda slopy with no PRIMARY KEY

CREATE TABLE Table3
(
	id   INTEGER NOT NULL,
	dataValue varchar(100) NOT NULL
) ;


INSERT INTO Table3 (id, DataValue) VALUES
    (1, 'Pizza'),(2, 'Burger'),(3, 'Sushi');

INSERT INTO Table2 (id, FavoriteColor) VALUES
    (1, 'red'),(2, 'green'),(3, 'blue'),
    (4, 'pink'),(5, 'purple'),(6, 'mauve'),
    (7, 'orange'),(8, 'yellow'),(1, 'indigo');

INSERT INTO Table1 (Name, fk, fk_table3) VALUES
    ('Steve', 1, NULL),('Aaron', 3, NULL),('Mary', 2, NULL),
    ('Fred', 1, NULL),('Anne', 5, NULL),('Beth', 8, 1),
    ('Johnny', NULL, 1), ('Karen', NULL, 2);


-- SELECT FROM THREE TABLES
SELECT *
  FROM Table1;

SELECT * 
  FROM Table2;

SELECT * 
  FROM Table3;




-- INNER JOIN
-- Show favorite colors
SELECT *
  FROM Table1 t1
 INNER JOIN Table2 t2
    ON t1.fk = t2.id;

-- INNER JOIN includes those items where the JOIN predicate is an exact match.
-- What about Johnny and Karen?


    
    
-- LEFT OUTER JOIN
SELECT *
  FROM Table1 t1
  LEFT OUTER JOIN Table2 t2
    ON t1.fk = t2.id;
    
-- OUTER JOIN (left) returns everything from the SELECT table, if there is a match on the JOIN table, then show it if not show nulls.





-- WHAT ABOUT THIS... 
SELECT *
  FROM Table1 t1
  LEFT JOIN Table2 t2
    ON t1.fk = t2.id;

    
-- It is a LEFT OUTER JOIN, but the intent of your query is not shown.



    
-- RIGHT OUTER JOIN
SELECT *
  FROM Table1 t1
 RIGHT OUTER JOIN Table2 t2
    ON t1.fk = t2.id;
-- OUTER JOIN (right) returns everything from the JOIN table, if there is match in the SELECT table, then show it, otherwise show nulls.
    
    


-- WHAT ABOUT THIS... 
SELECT *
  FROM Table1 t1
 RIGHT JOIN Table2 t2
    ON t1.fk = t2.id;




-- It is a RIGHT OUTER JOIN, but the intent of your query is not shown.



    
    
-- WHAT ABOUT THIS???
SELECT *
  FROM Table1 t1
  OUTER JOIN Table2 t2
    ON t1.fk = t2.id;
    
    
    
-- SEMI JOIN 
-- similar to an inner join with less duplication
SELECT *
  FROM Table1 t1
 WHERE EXISTS (SELECT 1
                 FROM Table2 t2
		 	    WHERE t1.fk = t2.id
			   ); 


               
-- ANTI SEMI JOIN
SELECT *
  FROM Table1 t1
 WHERE NOT EXISTS (SELECT 1
                     FROM Table2 t2
		 	        WHERE t1.fk = t2.id
			       ); 
                   


-- LEFT OUTER JOIN with Exclusion
SELECT *
  FROM Table1 t1
  LEFT OUTER JOIN Table2 t2
    ON t1.fk = t2.id
 WHERE t2.id is null;         
 
 
-- RIGHT OUTER JOIN with Exclusion
SELECT *
  FROM Table1 t1
  RIGHT OUTER JOIN Table2 t2
    ON t1.fk = t2.id
 WHERE t1.fk is null;         
 
 


-- FULL OUTER JOIN - not directly available on MySQL
    
SELECT *
  FROM Table1 t1
  LEFT OUTER JOIN Table2 t2 ON t1.fk = t2.id
UNION
SELECT *
  FROM Table1 t1 
 RIGHT OUTER JOIN Table2 t2 ON t1.fk = t2.id;



-- FULL OUTER JOIN with exclusions
    
SELECT *
  FROM Table1 t1
  LEFT OUTER JOIN Table2 t2 ON t1.fk = t2.id
 WHERE t2.id IS NULL
UNION
SELECT *
  FROM Table1 t1
 RIGHT OUTER JOIN Table2 t2 ON t1.fk = t2.id 
 WHERE t1.ID IS NULL;



-- TWO INNER JOINS

SELECT *
  FROM Table1 t1
 INNER JOIN Table2 t2
    ON t1.fk = t2.id
 INNER JOIN Table3 t3
    ON t1.fk_table3 = t3.id;
    
    
-- Two LEFT OUTER JOINs    
SELECT *
  FROM Table1 t1
  LEFT OUTER JOIN Table2 t2
    ON t1.fk = t2.id
  LEFT OUTER JOIN Table3 t3
    ON t1.fk_table3 = t3.id;
    